setwd("C:/Users/Austin/Documents/R/ABCDE/Paper2")
source("ABCDE_SCE.R")
ABCDDays <- read.table("Days_For_ABCDE.txt",header=TRUE)
ABCD_Met <- read.table("ABCD_Scituate.txt",header=FALSE)
len <- 132  #Run length - 11 years right now
Obs <- read.table("Scituate_Inflow.txt",header=TRUE)


days <- ABCDDays$Days
seasons <- ABCDDays$Seasons
Tavg <- rowMeans(ABCD_Met[553:684,2:3])
Prcp <- ABCD_Met[553:684,1]
PE <- ABCD_Met[553:684,4]
basin_area <- c(4.96,14.0)

#we convert streamflow to mm
cfs_to_cmms <- 28316846.6 #this converts cubic feet per second to cubic millimeters per second
squaremi_to_sqmm <-  2589988110336 #this converts square miles to square millimeters
sec_to_month <- 86400*days #this converts seconds to months
Qobs <- Obs[,4]*cfs_to_cmms*(1/squaremi_to_sqmm)*sec_to_month[553:684]*(1/basin_area[2]) #This is the streamflow

##Model initial conditions and parameters
Sint <- 225
Gint <- 10
Aint <- 20

###Calibrate Using SCE#######################################

##Calib Num is length of run
#set initial parameter values
start_par <- c(.98,300,.3,.2,.8,0,1)

#set lower and upper bounds on possible parameter values
lowerb <- c(0.80,100,0,0,0,-10,0)   
upperb <- c(1,700,1,1,1,10,10)   

ABCDE_SCE_CALIBRATE <- function(parameters,P,T,PE,Qobs,Sint,Gint,Aint,calib_num) {
A <- array(0,calib_num)
S <- array(0,calib_num)
G <- array(0,calib_num)
y.hat <- array(0,calib_num)
Y <- array(0,calib_num)
W <- array(0,calib_num)
E <- array(0,calib_num)
mt <- array(0,calib_num)
Peff <- array(0,calib_num)
ETeff <- array(0,calib_num)
SNOW_FRAC <- array(0,calib_num)

a <- parameters[1]
b <- parameters[2]
c1 <- parameters[3]
d <- parameters[4]
e <- parameters[5]
f <- parameters[6]
dif <- parameters[7]

A[1] <- Aint
S[1] <- Sint
G[1] <- Gint
y.hat[1] <- Qobs[1]
Y[1] <- 0
W[1] <- 0
E[1] <- 0
mt[1] <- 0
Peff[1] <- P[1]
ETeff[1] <- ET[1]

for (i in 2:calib_num){
#Snow Model
SNOW_FRAC[i] <- (f - T[i])/(dif)
if(T[i] >= (f-dif) && T[i] <= f){
mt[i] <- (A[i-1] + P[i]*SNOW_FRAC[i])*e*(1-SNOW_FRAC[i])
Peff[i] <- (P[i]*(1 - SNOW_FRAC[i]) + mt[i])
ETeff[i] <- 0
A[i] <- (A[i-1] + P[i]*SNOW_FRAC[i]) - mt[i]
} else if(T[i] > f) {
mt[i] <- A[i-1]
Peff[i] <- P[i]+mt[i]
ETeff[i] <- ET[i]
A[i] <- A[i-1] - mt[i]
} else {  #T[i] < (f-dif)
mt[i] <- 0
Peff[i] <- 0
ETeff[i] <- 0
A[i] <- A[i-1]+P[i]
}
#Begin ABCD Model
W[i] <- Peff[i] + S[i-1]
Y[i] <- (W[i]+b)/(2*a) - sqrt(((W[i]+b)/(2*a))^2-W[i]*b/a)
S[i] <- Y[i]*exp(-1*ETeff[i]/b)
E[i] <- Y[i]*(1-exp(-1*(ETeff[i]/b)))
G[i] <- (G[i-1] + c1*(W[i]-Y[i]))/(1+d)
y.hat[i] <- ((1-c1)*(W[i]-Y[i])+d*G[i])
}

RMSE <- sqrt(sum((Qobs - y.hat)^2))				
return(RMSE)	
}

SCEoptim(ABCDE_SCE_CALIBRATE,start_par,P=Prcp,T=Tavg,PE=PE,Qobs=Qobs,Sint=Sint,Gint=Gint,Aint=Aint,calib_num=len,lower=lowerb,upper=upperb)

###Run just the ABCDE model with set parameters
parameters <- c(0.992,307,0.2,0.35,0.33,-1.6,14)

ABCDE <- function(parameters,T,P,ET,Sint,Gint,Aint,calib_num) {
A <- array(0,calib_num)
S <- array(0,calib_num)
G <- array(0,calib_num)
y.hat <- array(0,calib_num)
Y <- array(0,calib_num)
W <- array(0,calib_num)
E <- array(0,calib_num)
mt <- array(0,calib_num)
Peff <- array(0,calib_num)
ETeff <- array(0,calib_num)
SNOW_FRAC <- array(0,calib_num)

a <- parameters[1]
b <- parameters[2]
c1 <- parameters[3]
d <- parameters[4]
e <- parameters[5]
f <- parameters[6]
dif <- parameters[7]

A[1] <- Aint
S[1] <- Sint
G[1] <- Gint
y.hat[1] <- Qobs[1]
Y[1] <- 0
W[1] <- 0
E[1] <- 0
mt[1] <- 0
Peff[1] <- P[1]
ETeff[1] <- ET[1]

for (i in 2:calib_num){
#Snow Model
SNOW_FRAC[i] <- (f - T[i])/(dif)
if(T[i] >= (f-dif) && T[i] <= f){
mt[i] <- (A[i-1] + P[i]*SNOW_FRAC[i])*e*(1-SNOW_FRAC[i])
Peff[i] <- (P[i]*(1 - SNOW_FRAC[i]) + mt[i])
ETeff[i] <- 0
A[i] <- (A[i-1] + P[i]*SNOW_FRAC[i]) - mt[i]
} else if(T[i] > f) {
mt[i] <- A[i-1]
Peff[i] <- P[i]+mt[i]
ETeff[i] <- ET[i]
A[i] <- A[i-1] - mt[i]
} else {  #T[i] < (f-dif)
mt[i] <- 0
Peff[i] <- 0
ETeff[i] <- 0
A[i] <- A[i-1]+P[i]
}
#Begin ABCD Model
W[i] <- Peff[i] + S[i-1]
Y[i] <- (W[i]+b)/(2*a) - sqrt(((W[i]+b)/(2*a))^2-W[i]*b/a)
S[i] <- Y[i]*exp(-1*ETeff[i]/b)
E[i] <- Y[i]*(1-exp(-1*(ETeff[i]/b)))
G[i] <- (G[i-1] + c1*(W[i]-Y[i]))/(1+d)
y.hat[i] <- ((1-c1)*(W[i]-Y[i])+d*G[i])
}
return(y.hat)
}


modflow <- ABCDE(parameters,Tavg,Prcp,PE,Sint,Gint,Aint,len)

plot(modflow,Qobs,xlim=c(0,200),ylim=c(0,200))
abline(0,1)

plot(modflow,type="l",lty=2,col="red")
lines(Qobs)

